@@ -269,13 +269,26 @@ Variable(
     added_in_version="before_v9",
     text=r"""
   * 0 --> the dipole-dipole interaction is not handled separately in the treatment of the interatomic forces.
-    This option is available for testing purposes or if effective charge and/or dielectric tensor is not available
-    in the derivative database. It gives results much less accurate than **dipdip** =1.
+    This option is available for testing purposes or if effective charge and/or dielectric tensor are not available
+    in the DDB file. In semiconductors, this option  gives results much less accurate than **dipdip** = 1.
+    Using this value in metals is fine since the interatomic forces are usually short-ranged.
+
   * 1 --> the dipole-dipole interaction is subtracted from the dynamical matrices before Fourier transform,
     so that only the short-range part is handled in real space. Of course, it is reintroduced analytically
     when the phonon spectrum is interpolated, or if the interatomic force constants have to be analysed in real space.
+    Note that the Ewald contribution to the dynamical matrix consists of two terms: a G-space sum and
+    and another (expensive) sum in R-space.
+    See Phys. Rev. B 55, 10355 (1997) [[cite:Gonze1997a]], equations (71) to (75).
+    This approach is very accurate but the computation of the R-space sum is rather expensive, especially
+    when one has to interpolate many q-points e.g. for PHDOS computations.
+    See dipdip -1 option below for a faster algorithm.
 
-The abinit input variable [[dipdip]] has a similar meaning.
+  * -1 --> Similar to +1 with the difference that the code computes a material-dependent width for
+    the Gaussians that will hopefully make the Ewald real-space summation unnecessary.
+    This option is **much faster** that dipdip 1 and is activated by default when
+    [[anaddb:dipquad]] or [[anaddb:quadquad]] are set to 1.
+    It is recommended to check that calculations with dipdip = 1 and -1 (both with dipquad = 0 and quadquad = 0)
+    lead to identical results. Otherwise increase the resolution of the q-point grid and repeat this test.
 """,
 ),
 
@@ -285,17 +298,23 @@ Variable(
     vartype="integer",
     topics=['longwave_basic'],
     dimensions="scalar",
-    defaultval=0,
+    defaultval=1,
     mnemonics="DIPole-QUADdrupole interaction",
     characteristics=['[[DEVELOP]]'],
     added_in_version="v9",
     text=r"""
-  * 0 --> the dipole-quadrupole interaction is not handled separately in the treatment of the interatomic forces.
-  * 1 --> the dipole-quadrupole interaction is subtracted from the dynamical matrices before Fourier transform,
-    so that only the short-range part is handled in real space. Of course, it is reintroduced analytically
-    when the phonon spectrum is interpolated. Requires a preceding generation of 3rd order DDB with a [[lw_qdrpl]] = 1
-    or a [[lw_flexo]] = 1 or 2 run.
+* 0 --> the dipole-quadrupole interaction is not handled separately in the treatment of the interatomic forces.
+* 1 --> the dipole-quadrupole interaction is subtracted from the dynamical matrices before Fourier transform,
+  so that only the short-range part is handled in real space. Of course, it is reintroduced analytically
+  when the phonon spectrum is interpolated. Requires a preceding generation of 3rd order DDB with a [[lw_qdrpl]] = 1
+  or a [[lw_flexo]] = 1 or 2 run.
 
+!!! important
+
+    The default value is 1.
+    This means that the dipole-quadrupole interaction is always included **provided** the DDB file contains
+    the dynamical quadrupoles.
+    If the DDB file does not contain the dynamical quadrupoles, this variable is automatically set to zero at runtime.
 """,
 ),
 
@@ -305,17 +324,23 @@ Variable(
     vartype="integer",
     topics=['longwave_basic'],
     dimensions="scalar",
-    defaultval=0,
+    defaultval=1,
     mnemonics="QUADdrupole-QUADdrupole interaction",
     characteristics=['[[DEVELOP]]'],
     added_in_version="v9",
     text=r"""
-  * 0 --> the quadrupole-quadrupole interaction is not handled separately in the treatment of the interatomic forces.
-  * 1 --> the quadrupole-quadrupole interaction is subtracted from the dynamical matrices before Fourier transform,
-    so that only the short-range part is handled in real space. Of course, it is reintroduced analytically
-    when the phonon spectrum is interpolated. Requires a preceding generation of 3rd order DDB with a [[lw_qdrpl]] = 1
-    or a [[lw_flexo]] = 1 or 2 run.
+* 0 --> the quadrupole-quadrupole interaction is not handled separately in the treatment of the interatomic forces.
+* 1 --> the quadrupole-quadrupole interaction is subtracted from the dynamical matrices before Fourier transform,
+  so that only the short-range part is handled in real space. Of course, it is reintroduced analytically
+  when the phonon spectrum is interpolated. Requires a preceding generation of 3rd order DDB with a [[lw_qdrpl]] = 1
+  or a [[lw_flexo]] = 1 or 2 run.
 
+!!! important
+
+    The default value is 1.
+    This means that the quadrupole-quadrupole interaction is always included **provided** the DDB file contains
+    the dynamical quadrupoles.
+    If the DDB file does not contain the dynamical quadrupoles, this variable is automatically set to zero at runtime.
 """,
 ),
 
