@@ -9,7 +9,6 @@ from pprint import pprint
 from monty.functools import lazy_property
 from monty.string import marquee
 
-
 from pymatgen.ext.matproj import MPRester, MPRestError
 from abipy.tools.printing import print_dataframe
 from abipy.core.mixins import NotebookWriter
@@ -42,7 +41,6 @@ def get_mprester(api_key=None, endpoint=None):
     if api_key is None:
         try:
             from pymatgen.core import SETTINGS
-            #from pymatgen.settings import SETTINGS
         except ImportError:
             from pymatgen import SETTINGS
 
@@ -285,7 +283,9 @@ class MpStructures(DatabaseStructures):
 
     @lazy_property
     def dataframe(self):
-        """Pandas dataframe constructed from self.data. None if data is not available."""
+        """
+        Pandas dataframe constructed from self.data. None if data is not available.
+        """
         if not self.data: return None
         import pandas as pd
         rows = []
