@@ -4,21 +4,28 @@ Brillouin zone
 ==============
 
 This example shows how to display the Brillouin zone
-with pymatgen and matplotlib.
+with matplotlib or plotly.
 """
+
+#%%
+# Open the WKF file and extract the crystalline structure.
+
 from abipy.abilab import abiopen
 import abipy.data as abidata
 
-# Open the WKF file.
 wfk_file = abiopen(abidata.ref_file("si_scf_WFK.nc"))
-
-# Extract the crystalline structure.
 structure = wfk_file.structure
 
 #%%
 # To visualize the BZ with matplotlib, use:
+
 structure.plot_bz()
 
+#%%
+# For the plotly version, use:
+
+structure.plotly_bz()
+
 #%%
 # Remember to close the wfk file with:
 wfk_file.close()
