@@ -15,8 +15,9 @@ ncfile = abiopen(abidata.ref_file("si_DEN.nc"))
 # The DEN file has a `Density`, a `Structure` and an `ElectronBands` object
 print(ncfile.structure)
 
+#%%
 # To plot the KS eigenvalues.
-#ncfile.ebands.plot()
+ncfile.ebands.plot()
 
 density = ncfile.density
 print(density)
@@ -24,14 +25,17 @@ print(density)
 # To visualize the total charge wih vesta
 #visu = density.visualize("vesta"); visu()
 
+#%%
 # To plot the density along the line connecting
 # the first and the second in the structure:
 density.plot_line(point1=0, point2=1)
 
+#%%
 # alternatively, one can define the line in terms of two points
 # in fractional coordinates:
 density.plot_line(point1=[0, 0, 0], point2=[2.25, 2.25, 2.25], num=300)
 
+#%%
 # To plot the density along the lines connect the firt atom in the structure
 # and all the neighbors within a sphere of radius 3 Angstrom:
 density.plot_line_neighbors(site_index=0, radius=3)
