@@ -1,9 +1,9 @@
 #!/usr/bin/env python
 r"""
-Seaborn boxplot
-===============
+Electron bands box plot
+=======================
 
-This example shows how to use seaborn to draw a box plot
+This example shows how to use seaborn or plotly to draw a box plot
 showing the distributions of eigenvalues with respect to the band index.
 """
 from abipy.abilab import abiopen
@@ -15,7 +15,9 @@ with abiopen(abidata.ref_file("si_scf_GSR.nc")) as gsr:
     ebands = gsr.ebands
 
 #%%
-import matplotlib.pyplot as plt
 # Use `swarm=True` to show the datapoints on top of the boxes
 ebands.boxplot(swarm=True)
-plt.show()
+
+#%%
+# For the plotly version, use:
+ebands.boxplotly()
