@@ -24,9 +24,12 @@ print(frame)
 
 #%%
 # To create a grid plot use:
-
 plotter.gridplot(with_gaps=True)
 
+#%%
+# Plotly version:
+plotter.gridplotly(with_gaps=True)
+
 #%%
 # To plot a grid with band structures + DOS, use the optional argument `edos_objects`
 # The first subplot gets the band dispersion from eb_objects[0] and the DOS from edos_objects[0]
@@ -41,3 +44,7 @@ plotter.add_ebands("Si", ref_file("si_nscf_GSR.nc"), edos=ref_file("si_scf_GSR.n
 plotter.add_ebands("Same data", ref_file("si_nscf_GSR.nc"), edos=ref_file("si_scf_GSR.nc"))
 # sphinx_gallery_thumbnail_number = 2
 plotter.gridplot()
+
+#%%
+# Plotly version:
+plotter.gridplotly()
