@@ -7,9 +7,13 @@ This example shows how to compute the speed of sound by fitting phonon frequenci
 along selected directions by linear least-squares fit.
 For a command line interface use:
 
-    abiview.py ddb_vs input_DDB
+.. code-block:: bash
+
+    abiview.py ddb_vs DDB_FILE
 """
-from __future__ import print_function
+
+#%%
+# Initialize object from DDB file.
 
 import os
 import abipy.data as abidata
@@ -17,14 +21,21 @@ import abipy.data as abidata
 from abipy import abilab
 from abipy.dfpt.vsound import SoundVelocity
 
-# Initialize object from DDB file.
 ddb_path = os.path.join(abidata.dirpath, "refs", "si_sound_vel", "Si_DDB")
 sv = SoundVelocity.from_ddb(ddb_path)
 
+#%%
 # Get pandas dataframe with results.
+
 df = sv.get_dataframe()
 abilab.print_dataframe(df)
 
-# Plot fit
+#%%
+# Plot fit with matplotlib
+
 sv.plot()
-sv.plotly()
+
+#%%
+# Plot fit with plotly
+
+sv.plotly(template="plotly_dark")
