@@ -85,7 +85,7 @@ class EffMassDFPTWork(Work):
     """
 
     @classmethod
-    def from_scf_input(cls, scf_input, k0_list, effmass_bands_f90, den_node=None, manager=None):
+    def from_scf_input(cls, scf_input, k0_list, effmass_bands_f90, ngfft=None, den_node=None, manager=None):
         """
         Build the Work from an |AbinitInput| representing a GS-SCF calculation.
 
@@ -94,12 +94,13 @@ class EffMassDFPTWork(Work):
             k0_list: List with the reduced coordinates of the k-points where effective masses are wanted.
             effmass_bands_f90: (nkpt, 2) array with band range for effmas computation.
                 WARNING: Assumes Fortran convention with indices starting from 1.
+            ngfft: FFT divisions (3 integers). Used to enforce the same FFT mesh in the NSCF run as the one used for GS.
             den_node: Path to the DEN file or Task object producing a DEN file.
                 Can be used to avoid the initial SCF calculation if a DEN file is already available.
                 If None, a GS calculation is performed.
             manager: |TaskManager| instance. Use default if None.
         """
-        multi = scf_input.make_dfpt_effmass_inputs(k0_list, effmass_bands_f90)
+        multi = scf_input.make_dfpt_effmass_inputs(k0_list, effmass_bands_f90, ngfft=ngfft)
         nscf_input, effmass_input = multi[0], multi[1]
 
         new = cls(manager=manager)
@@ -128,14 +129,21 @@ class EffMassAutoDFPTWork(Work):
     """
 
     @classmethod
-    def from_scf_input(cls, scf_input, ndivsm=15, tolwfr=1e-20, manager=None):
+    def from_scf_input(cls, scf_input, ndivsm=15, tolwfr=1e-20, den_node=None, manager=None):
         """
         Build the Work from an |AbinitInput| representing a GS-SCF calculation.
 
         Args:
             scf_input: |AbinitInput| for GS-SCF used as template to generate the other inputs.
-            ndivsm: Number of divisions used to sample the smallest segment of the k-path.
+            ndivsm: if > 0, it's the number of divisions for the smallest segment of the path (Abinit variable).
+                if < 0, it's interpreted as the pymatgen `line_density` parameter in which the number of points
+                in the segment is proportional to its length. Typical value: -20.
+                This option is the recommended one if the k-path contains two high symmetry k-points that are very close
+                as ndivsm > 0 may produce a very large number of wavevectors.
             tolwfr: Tolerance on residuals for NSCF calculation
+            den_node: Path to the DEN file or Task object producing a DEN file.
+                Can be used to avoid the initial SCF calculation if a DEN file is already available.
+                If None, a GS calculation is performed.
             manager: |TaskManager| instance. Use default if None.
         """
         if scf_input.get("nsppol", 1) != 1:
@@ -146,11 +154,14 @@ class EffMassAutoDFPTWork(Work):
         new.scf_input = scf_input.deepcopy()
 
         # Need SCF run to get DEN file.
-        new.scf_task = new.register_scf_task(new.scf_input)
+        if den_node is None:
+            new.den_node = new.register_scf_task(new.scf_input)
+        else:
+            new.den_node = Node.as_node(den_node)
 
         # Perform NSCF run along k-path that will be used to find band extrema.
         bands_input = scf_input.make_ebands_input(ndivsm=ndivsm, tolwfr=tolwfr)
-        new.bands_task = new.register_nscf_task(bands_input, deps={new.scf_task: "DEN"})
+        new.bands_task = new.register_nscf_task(bands_input, deps={new.den_node: "DEN"})
 
         return new
 
@@ -167,10 +178,12 @@ class EffMassAutoDFPTWork(Work):
             ebands.set_fermie_to_vbm()
             # Find k0_list and effmass_bands_f90
             k0_list, effmass_bands_f90 = ebands.get_kpoints_and_band_range_for_edges()
+            den_ngfft = gsr.reader.read_ngfft3()
 
         # Create the work for effective mass computation with DFPT and add it to the flow.
         # Keep a reference in generated_effmass_dfpt_work.
-        work = EffMassDFPTWork.from_scf_input(self.scf_input, k0_list, effmass_bands_f90, den_node=self.scf_task)
+        work = EffMassDFPTWork.from_scf_input(self.scf_input, k0_list, effmass_bands_f90,
+                                              ngfft=den_ngfft, den_node=self.den_node)
 
         self.generated_effmass_dfpt_work = work
         self.flow.register_work(work)
