@@ -2,6 +2,7 @@
 """
 Works for Abinit
 """
+from __future__ import annotations
 
 import os
 import shutil
@@ -9,17 +10,21 @@ import time
 import abc
 import collections
 import numpy as np
+import pandas as pd
 
+#from typying import List, Any
 from monty.collections import AttrDict
 from monty.itertools import chunks
 from monty.functools import lazy_property
 from monty.fnmatch import WildCard
+from monty.dev import deprecated
 from pydispatch import dispatcher
 from pymatgen.core.units import EnergyArray
 from . import wrappers
-from .nodes import Dependency, Node, NodeError, NodeResults, FileNode #, check_spectator
-from .tasks import (Task, AbinitTask, ScfTask, NscfTask, DfptTask, PhononTask, ElasticTask, DdkTask, EffMassTask,
-                    BseTask, RelaxTask, DdeTask, BecTask, ScrTask, SigmaTask, TaskManager,
+from .nodes import Dependency, Node, NodeError, NodeResults, FileNode, Status #, check_spectator
+from .tasks import (Task, AbinitTask, ScfTask, NscfTask, DfptTask, PhononTask, ElasticTask, DdkTask,
+                    DkdkTask, QuadTask, FlexoETask, DdeTask, BecTask,
+                    EffMassTask, BseTask, RelaxTask, ScrTask, SigmaTask, TaskManager,
                     DteTask, EphTask, KerangeTask, CollinearThenNonCollinearScfTask)
 
 from .utils import Directory
@@ -119,7 +124,7 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
         return [task.returncode for task in self]
 
     @property
-    def ncores_reserved(self):
+    def ncores_reserved(self) -> int:
         """
         Returns the number of cores reserved in this moment.
         A core is reserved if it's still not running but
@@ -128,7 +133,7 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
         return sum(task.manager.num_cores for task in self if task.status == task.S_SUB)
 
     @property
-    def ncores_allocated(self):
+    def ncores_allocated(self) -> int:
         """
         Returns the number of CPUs allocated in this moment.
         A core is allocated if it's running a task or if we have
@@ -137,7 +142,7 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
         return sum(task.manager.num_cores for task in self if task.status in [task.S_SUB, task.S_RUN])
 
     @property
-    def ncores_used(self):
+    def ncores_used(self) -> int:
         """
         Returns the number of cores used in this moment.
         A core is used if there's a job that is running on it.
@@ -180,7 +185,7 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
     def _setup(self, *args, **kwargs):
         self.setup(*args, **kwargs)
 
-    def connect_signals(self):
+    def connect_signals(self) -> None:
         """
         Connect the signals within the work.
         The |Work| is responsible for catching the important signals raised from
@@ -189,9 +194,10 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
         for task in self:
             dispatcher.connect(self.on_ok, signal=task.S_OK, sender=task)
 
-    def disconnect_signals(self):
+    def disconnect_signals(self) -> None:
         """
-        Disable the signals within the work. This function reverses the process of `connect_signals`
+        Disable the signals within the work.
+        This function reverses the process of `connect_signals`
         """
         for task in self:
             try:
@@ -200,7 +206,7 @@ class BaseWork(Node, metaclass=abc.ABCMeta):
                 self.history.debug(str(exc))
 
     @property
-    def all_ok(self):
+    def all_ok(self) -> bool:
         return all(task.status == task.S_OK for task in self)
 
     #@check_spectator
@@ -369,8 +375,8 @@ class NodeContainer(metaclass=abc.ABCMeta):
         # TODO: shall flow.register_task return a Task or a Work?
 
     # Helper functions to register Task subclasses.
-    def register_scf_task(self, *args, **kwargs):
-        """Register a Scf task."""
+    def register_scf_task(self, *args, **kwargs) -> ScfTask:
+        """Register a SCF task."""
         kwargs["task_class"] = ScfTask
         return self.register_task(*args, **kwargs)
 
@@ -379,12 +385,12 @@ class NodeContainer(metaclass=abc.ABCMeta):
         kwargs["task_class"] = CollinearThenNonCollinearScfTask
         return self.register_task(*args, **kwargs)
 
-    def register_nscf_task(self, *args, **kwargs):
-        """Register a nscf task."""
+    def register_nscf_task(self, *args, **kwargs) -> NscfTask:
+        """Register a NSCF task."""
         kwargs["task_class"] = NscfTask
         task = self.register_task(*args, **kwargs)
 
-        # Make sure parent producing DEN file is given
+        # Make sure the parent producing the DEN file is given.
         if task.is_work: task = task[-1]
         den_parent = task.find_parent_with_ext("DEN")
         if den_parent is None:
@@ -392,7 +398,7 @@ class NodeContainer(metaclass=abc.ABCMeta):
 
         if task.input.get("usekden", 0) == 1:
             # Meta-GGA calculation --> Add KDEN if not explicitly given.
-            # Assuming prtkden already set to 1
+            # Assuming prtkden is already set to 1
             # TODO: Abinit should automatically set it to 1 if usekden --> I'm not gonna fix the input at this level
             kden_parent = task.find_parent_with_ext("KDEN")
             if kden_parent is None:
@@ -400,83 +406,110 @@ class NodeContainer(metaclass=abc.ABCMeta):
 
         return task
 
-    def register_relax_task(self, *args, **kwargs):
+    def register_relax_task(self, *args, **kwargs) -> RelaxTask:
         """Register a task for structural optimization."""
         kwargs["task_class"] = RelaxTask
         return self.register_task(*args, **kwargs)
 
-    def register_phonon_task(self, *args, **kwargs):
+    def register_phonon_task(self, *args, **kwargs) -> PhononTask:
         """Register a phonon task."""
         kwargs["task_class"] = PhononTask
         return self.register_task(*args, **kwargs)
 
-    def register_elastic_task(self, *args, **kwargs):
+    def register_elastic_task(self, *args, **kwargs) -> ElasticTask:
         """Register an elastic task."""
         kwargs["task_class"] = ElasticTask
         return self.register_task(*args, **kwargs)
 
-    def register_ddk_task(self, *args, **kwargs):
+    def register_ddk_task(self, *args, **kwargs) -> DdkTask:
         """Register a DDK task."""
         kwargs["task_class"] = DdkTask
         return self.register_task(*args, **kwargs)
 
-    def register_effmass_task(self, *args, **kwargs):
+    def register_dkdk_task(self, *args, **kwargs) -> DkdkTask:
+        """Register a DkdkTask task."""
+        kwargs["task_class"] = DkdkTask
+        return self.register_task(*args, **kwargs)
+
+    def register_quad_task(self, *args, **kwargs) -> QuadTask:
+        """Register a QuadTask task."""
+        kwargs["task_class"] = QuadTask
+        return self.register_task(*args, **kwargs)
+
+    def register_flexoe_task(self, *args, **kwargs) -> FlexoETask:
+        """Register a FlexETask task."""
+        kwargs["task_class"] = FlexoETask
+        return self.register_task(*args, **kwargs)
+
+    def register_effmass_task(self, *args, **kwargs) -> EffMassTask:
         """Register a effective mass task."""
         kwargs["task_class"] = EffMassTask
         # FIXME: Hack to run it in sequential because effmass task does not support parallelism.
         kwargs.update({"manager": TaskManager.from_user_config().new_with_fixed_mpi_omp(1, 1)})
         return self.register_task(*args, **kwargs)
 
-    def register_scr_task(self, *args, **kwargs):
+    def register_scr_task(self, *args, **kwargs) -> ScrTask:
         """Register a screening task."""
         kwargs["task_class"] = ScrTask
         return self.register_task(*args, **kwargs)
 
-    def register_sigma_task(self, *args, **kwargs):
+    def register_sigma_task(self, *args, **kwargs) -> SigmaTask:
         """Register a sigma task."""
         kwargs["task_class"] = SigmaTask
         return self.register_task(*args, **kwargs)
 
-    def register_dde_task(self, *args, **kwargs):
+    def register_dde_task(self, *args, **kwargs) -> DdeTask:
         """Register a Dde task."""
         kwargs["task_class"] = DdeTask
         return self.register_task(*args, **kwargs)
 
-    def register_dte_task(self, *args, **kwargs):
+    def register_dte_task(self, *args, **kwargs) -> DteTask:
         """Register a Dte task."""
         kwargs["task_class"] = DteTask
         return self.register_task(*args, **kwargs)
 
-    def register_bec_task(self, *args, **kwargs):
+    def register_bec_task(self, *args, **kwargs) -> BecTask:
         """Register a BEC task."""
         kwargs["task_class"] = BecTask
         return self.register_task(*args, **kwargs)
 
-    def register_bse_task(self, *args, **kwargs):
+    def register_bse_task(self, *args, **kwargs) -> BseTask:
         """Register a Bethe-Salpeter task."""
         kwargs["task_class"] = BseTask
         return self.register_task(*args, **kwargs)
 
-    def register_eph_task(self, *args, **kwargs):
+    def register_eph_task(self, *args, **kwargs) -> EphTask:
         """Register an electron-phonon task."""
         kwargs["task_class"] = EphTask
         eph_inp = args[0]
         if eph_inp.get("eph_frohlichm", 0) != 0 or abs(eph_inp.get("eph_task", 0)) == 15:
-            # FIXME: Hack to run task in sequential since this  calculation does
+            # FIXME: Hack to run task in sequential since this calculation does
             # not support MPI with nprocs > 1.
             seq_manager = TaskManager.from_user_config().new_with_fixed_mpi_omp(1, 1)
             kwargs.update({"manager": seq_manager})
 
-        if eph_inp.get("eph_task",0) == -4:
-            max_cores = TaskManager.from_user_config().qadapter.max_cores
+        if eph_inp.get("eph_task", 0) == -4:
+            manager = TaskManager.from_user_config()
+            max_cores = manager.qadapter.max_cores
+            min_cores = manager.qadapter.min_cores
             natom3 = 3 * len(eph_inp.structure)
-            nprocs = max(max_cores - max_cores % natom3, 1)
-            new_manager = TaskManager.from_user_config().new_with_fixed_mpi_omp(nprocs, 1)
+            nprocs = max(max_cores - max_cores % natom3, min_cores)
+            new_manager = manager.new_with_fixed_mpi_omp(nprocs, 1)
+            kwargs.update({"manager": new_manager})
+
+        if eph_inp.get("eph_task", 0) == 9:
+            nkptgw = eph_inp.vars["nkptgw"]
+            manager = TaskManager.from_user_config()
+            max_cores = manager.qadapter.max_cores
+            nprocs = max_cores
+            if max_cores > nkptgw:
+                nprocs = nkptgw
+            new_manager = manager.new_with_fixed_mpi_omp(nprocs, 1)
             kwargs.update({"manager": new_manager})
 
         return self.register_task(*args, **kwargs)
 
-    def register_kerange_task(self, *args, **kwargs):
+    def register_kerange_task(self, *args, **kwargs) -> KerangeTask:
         """ Register a kerange task."""
         kwargs["task_class"] = KerangeTask
         seq_manager = TaskManager.from_user_config().new_with_fixed_mpi_omp(1, 1)
@@ -537,7 +570,7 @@ class Work(BaseWork, NodeContainer):
         if manager is not None:
             self.set_manager(manager)
 
-    def set_manager(self, manager):
+    def set_manager(self, manager: TaskManager):
         """Set the |TaskManager| to use to launch the |Task|."""
         self.manager = manager.deepcopy()
         for task in self:
@@ -548,7 +581,7 @@ class Work(BaseWork, NodeContainer):
         """The flow containing this |Work|."""
         return self._flow
 
-    def set_flow(self, flow):
+    def set_flow(self, flow) -> None:
         """Set the flow associated to this |Work|."""
         if not hasattr(self, "_flow"):
             self._flow = flow
@@ -558,18 +591,18 @@ class Work(BaseWork, NodeContainer):
 
     @lazy_property
     def pos(self):
-        """The position of self in the |Flow|"""
+        """The position of work in the |Flow|"""
         for i, work in enumerate(self.flow):
             if self == work:
                 return i
         raise ValueError("Cannot find the position of %s in flow %s" % (self, self.flow))
 
     @property
-    def pos_str(self):
+    def pos_str(self) -> str:
         """String representation of self.pos"""
         return "w" + str(self.pos)
 
-    def set_workdir(self, workdir, chroot=False):
+    def set_workdir(self, workdir: str, chroot=False) -> None:
         """Set the working directory. Cannot be set more than once unless chroot is True"""
         if not chroot and hasattr(self, "workdir") and self.workdir != workdir:
             raise ValueError("self.workdir != workdir: %s, %s" % (self.workdir,  workdir))
@@ -585,7 +618,7 @@ class Work(BaseWork, NodeContainer):
         self.tmpdir = Directory(os.path.join(self.workdir, "tmpdata"))
         self.wdir = Directory(self.workdir)
 
-    def chroot(self, new_workdir):
+    def chroot(self, new_workdir: str) -> None:
         self.set_workdir(new_workdir, chroot=True)
 
         for i, task in enumerate(self):
@@ -638,17 +671,17 @@ class Work(BaseWork, NodeContainer):
         return [task.process for task in self]
 
     @property
-    def all_done(self):
+    def all_done(self) -> bool:
         """True if all the |Task| objects in the |Work| are done."""
         return all(task.status >= task.S_DONE for task in self)
 
     @property
-    def isnc(self):
+    def isnc(self) -> bool:
         """True if norm-conserving calculation."""
         return all(task.isnc for task in self)
 
     @property
-    def ispaw(self):
+    def ispaw(self) -> bool:
         """True if PAW calculation."""
         return all(task.ispaw for task in self)
 
@@ -763,6 +796,17 @@ class Work(BaseWork, NodeContainer):
         self.outdir.makedirs()
         self.tmpdir.makedirs()
 
+        # Add README.md file if set
+        readme_md = getattr(self, "readme_md", None)
+        if readme_md is not None:
+            with open(self.path_in_workdir("README.md"), "wt") as fh:
+                fh.write(readme_md)
+
+        # Add abipy_meta.json file if set
+        data = getattr(self, "abipy_meta_json", None)
+        if data is not None:
+            self.write_json_in_workdir("abipy_meta.json", data)
+
         # Build dirs and files of each task.
         for task in self:
             task.build(*args, **kwargs)
@@ -771,7 +815,7 @@ class Work(BaseWork, NodeContainer):
         self.connect_signals()
 
     @property
-    def status(self):
+    def status(self) -> Status:
         """
         Returns the status of the work i.e. the minimum of the status of the tasks.
         """
@@ -813,7 +857,30 @@ class Work(BaseWork, NodeContainer):
             if task.status < task.S_SUB and all(status == task.S_OK for status in task.deps_status):
                 task.set_status(task.S_READY, "Status set to Ready")
 
-    def rmtree(self, exclude_wildcard=""):
+    def get_panel(self, **kwargs):
+        """
+        Build panel with widgets to interact with the Work either in a notebook or in panel app.
+        This is the implementation provided by the base class.
+        Subclasses may provide specialized implementations.
+        """
+        from abipy.panels.works import WorkPanel
+        return WorkPanel(work=self).get_panel(**kwargs)
+
+    def get_dataframe(self, as_dict=False) -> pd.DataFrame:
+        """
+        Return pandas dataframe task info or dictionary if as_dict is True.
+        This function should be called after work.get_status to update the status.
+        """
+        rows = []
+        for task in self:
+            d = task.get_dataframe(as_dict=True)
+            rows.append(d)
+
+        if as_dict: return rows
+
+        return pd.DataFrame(rows)
+
+    def rmtree(self, exclude_wildcard: str = "") -> None:
         """
         Remove all files and directories in the working directory
 
@@ -834,22 +901,22 @@ class Work(BaseWork, NodeContainer):
                     if not w.match(fname):
                         os.remove(path)
 
-    def rm_indatadir(self):
+    def rm_indatadir(self) -> None:
         """Remove all the indata directories."""
         for task in self:
             task.rm_indatadir()
 
-    def rm_outdatadir(self):
+    def rm_outdatadir(self) -> None:
         """Remove all the indata directories."""
         for task in self:
             task.rm_outatadir()
 
-    def rm_tmpdatadir(self):
+    def rm_tmpdatadir(self) -> None:
         """Remove all the tmpdata directories."""
         for task in self:
             task.rm_tmpdatadir()
 
-    def move(self, dest, isabspath=False):
+    def move(self, dest, isabspath=False) -> None:
         """
         Recursively move self.workdir to another location. This is similar to the Unix "mv" command.
         The destination path must not already exist. If the destination already exists
@@ -1244,7 +1311,7 @@ class BseMdfWork(Work):
 
     def get_mdf_robot(self):
         """Builds and returns a :class:`MdfRobot` for analyzing the results in the MDF files."""
-        from abilab.robots import MdfRobot
+        from abipy.electrons.bse import MdfRobot
         robot = MdfRobot()
         for task in self[2:]:
             mdf_path = task.outdir.has_abiext(robot.EXT)
@@ -1262,6 +1329,8 @@ class QptdmWork(Work):
     .. rubric:: Inheritance Diagram
     .. inheritance-diagram:: QptdmWork
     """
+
+    @deprecated(message="QptdmWork is deprecated and will be removed in abipy 1.0, use flowtk.ScreeningWork")
     def create_tasks(self, wfk_file, scr_input):
         """
         Create the SCR tasks and register them in self.
@@ -1344,10 +1413,11 @@ class QptdmWork(Work):
 
 class MergeDdb(object):
     """
-    Mixin class for Works that have to merge the DDB files produced by the tasks.
+    Mixin class for Works that need to merge the DDB files produced by the tasks in self.
     """
 
-    def add_becs_from_scf_task(self, scf_task, ddk_tolerance, ph_tolerance):
+    def add_becs_from_scf_task(self, scf_task, ddk_tolerance, ph_tolerance,
+                               with_quad=False, with_flexoe=False):
         """
         Build tasks for the computation of Born effective charges and add them to the work.
 
@@ -1355,8 +1425,12 @@ class MergeDdb(object):
             scf_task: |ScfTask| object.
             ddk_tolerance: dict {"varname": value} with the tolerance used in the DDK run. None to use AbiPy default.
             ph_tolerance: dict {"varname": value} with the tolerance used in the phonon run. None to use AbiPy default.
+            with_quad: Activate calculation of dynamical quadrupoles.
+                Note that only selected features are compatible with dynamical quadrupoles.
+                Please consult <https://docs.abinit.org/topics/longwave/>
+            with_flexoe: True to activate computation of flexoelectric tensor.
 
-        Return: (ddk_tasks, bec_tasks)
+        Return: (ddk_tasks, bec_tasks, dkdk_task, quad_task)
         """
         if not isinstance(scf_task, ScfTask):
             raise TypeError("task `%s` does not inherit from ScfTask" % scf_task)
@@ -1374,13 +1448,45 @@ class MergeDdb(object):
         bec_deps = {ddk_task: "DDK" for ddk_task in ddk_tasks}
         bec_deps.update({scf_task: "WFK"})
 
-        bec_inputs = scf_task.input.make_bec_inputs(tolerance=ph_tolerance)
+        if with_flexoe:
+            bec_inputs = scf_task.input.make_strain_perts_inputs(tolerance=ph_tolerance,
+                                                                 phonon_pert=True, efield_pert=True,
+                                                                 prepalw=1)
+
+        else:
+            bec_inputs = scf_task.input.make_bec_inputs(tolerance=ph_tolerance,
+                                                        prepalw=1 if with_quad else 0)
+
         bec_tasks = []
         for bec_inp in bec_inputs:
             bec_task = self.register_bec_task(bec_inp, deps=bec_deps)
             bec_tasks.append(bec_task)
 
-        return ddk_tasks, bec_tasks
+        if with_quad or with_flexoe:
+            # Response function calculation of d2/dkdk wave function.
+            # See <https://docs.abinit.org/tests/tutorespfn/Input/tlw_4.abi>
+            dkdk_inp = scf_task.input.make_dkdk_input(tolerance=ddk_tolerance)
+            dkdk_task = self.register_dkdk_task(dkdk_inp, deps=bec_deps)
+
+            quad_deps = bec_deps.copy()
+            quad_deps.update({dkdk_task: "DKDK"})
+            quad_deps.update({bec_task: ["1DEN", "1WF"] for bec_task in bec_tasks})
+
+            if with_quad:
+                # Dynamic Quadrupoles calculation
+                quad_inp = scf_task.input.make_quad_input(tolerance=ph_tolerance)
+                quad_task = self.register_quad_task(quad_inp, deps=quad_deps)
+
+            if with_flexoe:
+                #flexoe_inp = scf_task.input.make_flexoe_input(tolerance=ph_tolerance)
+                flexoe_inp = scf_task.input.new_with_vars(
+                    optdriver=10,
+                    lw_flexo=1,
+                    kptopt=2,
+                    useylm=1,
+                )
+
+                self.register_flexoe_task(flexoe_inp, deps=quad_deps)
 
     def merge_ddb_files(self, delete_source_ddbs=False, only_dfpt_tasks=True,
                         exclude_tasks=None, include_tasks=None):
@@ -1468,7 +1574,7 @@ class MergeDdb(object):
         out_dvdb = self.outdir.path_in("out_DVDB")
 
         if len(pot1_files) == 1:
-            # Avoid the merge. Just move the DDB file to the outdir of the work
+            # Avoid the merge. Just move the DVDB file to the outdir of the work
             shutil.copy(pot1_files[0], out_dvdb)
         else:
             # FIXME: The merge may require a non-negligible amount of memory if lots of qpts.
@@ -1493,8 +1599,10 @@ class PhononWork(Work, MergeDdb):
     """
 
     @classmethod
-    def from_scf_task(cls, scf_task, qpoints, is_ngqpt=False, tolerance=None, with_becs=False,
-                      ddk_tolerance=None, prtwf=-1, manager=None):
+    def from_scf_task(cls, scf_task, qpoints, is_ngqpt=False, with_becs=False,
+                      with_quad=False, with_flexoe=False, with_dvdb=True,
+                      tolerance=None, ddk_tolerance=None,
+                      prtwf=-1, manager=None) -> PhononWork:
         """
         Construct a `PhononWork` from a |ScfTask| object.
         The input file for phonons is automatically generated from the input of the ScfTask.
@@ -1505,9 +1613,15 @@ class PhononWork(Work, MergeDdb):
             qpoints: q-points in reduced coordinates. Accepts single q-point, list of q-points
                 or three integers defining the q-mesh if `is_ngqpt`.
             is_ngqpt: True if `qpoints` should be interpreted as divisions instead of q-points.
+            with_becs: Activate calculation of Electric field and Born effective charges.
+            with_quad: Activate calculation of dynamical quadrupoles. Require `with_becs`
+                Note that only selected features are compatible with dynamical quadrupoles.
+                Please consult <https://docs.abinit.org/topics/longwave/>
+            with_flexoe: True to activate computation of flexoelectric tensor. Require `with_becs`
+            with_dvdb: True to merge POT1 files associated to atomic perturbations in the DVDB file
+                at the end of the calculation
             tolerance: dict {"varname": value} with the tolerance to be used in the phonon run.
                 None to use AbiPy default.
-            with_becs: Activate calculation of Electric field and Born effective charges.
             ddk_tolerance: dict {"varname": value} with the tolerance used in the DDK run if with_becs.
                 None to use AbiPy default.
             prtwf: Controls the output of the first-order WFK.
@@ -1525,8 +1639,15 @@ class PhononWork(Work, MergeDdb):
         qpoints = np.reshape(qpoints, (-1, 3))
 
         new = cls(manager=manager)
+        new.with_dvdb = with_dvdb
+
+        if (with_quad or with_flexoe) and not with_becs:
+            raise RuntimeError("with_quad or with_flexoe require with_becs")
+
         if with_becs:
-            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance=tolerance)
+            # Special treatment of q == 0.
+            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance=tolerance,
+                                       with_quad=with_quad, with_flexoe=with_flexoe)
 
         for qpt in qpoints:
             is_gamma = np.sum(qpt ** 2) < 1e-12
@@ -1540,8 +1661,10 @@ class PhononWork(Work, MergeDdb):
         return new
 
     @classmethod
-    def from_scf_input(cls, scf_input, qpoints, is_ngqpt=False, tolerance=None,
-                       with_becs=False, ddk_tolerance=None, prtwf=-1, manager=None):
+    def from_scf_input(cls, scf_input, qpoints, is_ngqpt=False, with_becs=False,
+                       with_quad=False, with_flexoe=False,
+                       with_dvdb=True, tolerance=None,
+                       ddk_tolerance=None, prtwf=-1, manager=None) -> PhononWork:
         """
         Similar to `from_scf_task`, the difference is that this method requires
         an input for SCF calculation. A new |ScfTask| is created and added to the Work.
@@ -1553,11 +1676,18 @@ class PhononWork(Work, MergeDdb):
         qpoints = np.reshape(qpoints, (-1, 3))
 
         new = cls(manager=manager)
+        new.with_dvdb = with_dvdb
+
         # Create ScfTask
         scf_task = new.register_scf_task(scf_input)
 
+        if (with_quad or with_flexoe) and not with_becs:
+            raise RuntimeError("with_quad or with_flexoe require with_becs")
+
         if with_becs:
-            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance=tolerance)
+            # Special treatment of q == 0.
+            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance=tolerance,
+                                       with_quad=with_quad, with_flexoe=with_flexoe)
 
         for qpt in qpoints:
             is_gamma = np.sum(qpt ** 2) < 1e-12
@@ -1578,9 +1708,11 @@ class PhononWork(Work, MergeDdb):
         the final DDB file in the outdir of the |Work|.
         """
         # Merge DDB files.
-        out_ddb = self.merge_ddb_files()
-        # Merge DVDB files.
-        out_dvdb = self.merge_pot1_files()
+        out_ddb = self.merge_ddb_files(only_dfpt_tasks=False)
+
+        if getattr(self, "with_dvdb", True):
+            # Merge DVDB files (use getattr to maintain backward compability with pickle).
+            out_dvdb = self.merge_pot1_files()
 
         return self.Results(node=self, returncode=0, message="DDB merge done")
 
@@ -1600,8 +1732,9 @@ class PhononWfkqWork(Work, MergeDdb):
 
     @classmethod
     def from_scf_task(cls, scf_task, ngqpt, ph_tolerance=None, tolwfr=1.0e-22, nband=None,
-                      with_becs=False, ddk_tolerance=None, shiftq=(0, 0, 0), is_ngqpt=True, remove_wfkq=True,
-                      prepgkk=0, manager=None):
+                      with_becs=False, with_quad=False, ddk_tolerance=None, shiftq=(0, 0, 0),
+                      is_ngqpt=True, remove_wfkq=True,
+                      prepgkk=0, manager=None) -> PhononWfkqWork:
         """
         Construct a `PhononWfkqWork` from a |ScfTask| object.
         The input files for WFQ and phonons are automatically generated from the input of the ScfTask.
@@ -1611,6 +1744,9 @@ class PhononWfkqWork(Work, MergeDdb):
             scf_task: |ScfTask| object.
             ngqpt: three integers defining the q-mesh
             with_becs: Activate calculation of Electric field and Born effective charges.
+            with_quad: Activate calculation of dynamical quadrupoles.
+                Note that only selected features are compatible with dynamical quadrupoles.
+                Please consult <https://docs.abinit.org/topics/longwave/>
             ph_tolerance: dict {"varname": value} with the tolerance for the phonon run.
                 None to use AbiPy default.
             tolwfr: tolerance used to compute WFQ.
@@ -1646,7 +1782,7 @@ class PhononWfkqWork(Work, MergeDdb):
 
         if with_becs:
             # Add DDK and BECS.
-            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance)
+            new.add_becs_from_scf_task(scf_task, ddk_tolerance, ph_tolerance, with_quad=with_quad)
 
         # Get ngkpt, shift for electrons from input.
         # Won't try to skip WFQ if multiple shifts or off-diagonal kptrlatt
@@ -1680,6 +1816,7 @@ class PhononWfkqWork(Work, MergeDdb):
                 if nband:
                     nbdbuf = max(2, nband*0.1)
                     nscf_inp.set_vars(nband=nband+nbdbuf, nbdbuf=nbdbuf)
+
                 wfkq_task = new.register_nscf_task(nscf_inp, deps={scf_task: ["DEN", "WFK"]})
                 new.wfkq_tasks.append(wfkq_task)
 
@@ -1719,7 +1856,7 @@ class PhononWfkqWork(Work, MergeDdb):
         the final DDB file in the outdir of the |Work|.
         """
         # Merge DDB files.
-        out_ddb = self.merge_ddb_files()
+        out_ddb = self.merge_ddb_files(only_dfpt_tasks=True)
 
         # Merge DVDB files.
         out_dvdb = self.merge_pot1_files()
@@ -1760,7 +1897,7 @@ class GKKPWork(Work):
         # Create a WFK task
         kptopt = 1 if expand else 3
         nscf_inp = inp.new_with_vars(iscf=-2, kptopt=kptopt)
-        wfk_task = new.register_nscf_task(nscf_inp, deps={den_file: "DEN"},manager=tm)
+        wfk_task = new.register_nscf_task(nscf_inp, deps={den_file: "DEN"}, manager=tm)
         new.wfkq_tasks.append(wfk_task)
         new.wfk_task = wfk_task
 
@@ -1785,7 +1922,7 @@ class GKKPWork(Work):
             fbz_nscf_inp = inp.new_with_vars(optdriver=8)
             fbz_nscf_inp.set_spell_check(False)
             fbz_nscf_inp.set_vars(wfk_task="wfk_fullbz")
-            tm_serial = manager.new_with_fixed_mpi_omp(1,1)
+            tm_serial = manager.new_with_fixed_mpi_omp(1, 1)
             wfk_task = new.register_nscf_task(fbz_nscf_inp, deps={wfk_task: "WFK", den_file: "DEN"},
                                               manager=tm_serial)
             new.wfkq_tasks.append(wfk_task)
@@ -1793,7 +1930,7 @@ class GKKPWork(Work):
 
         if with_ddk:
             kptopt = 3 if expand else 1
-            ddk_inp = inp.new_with_vars(optdriver=8,kptopt=kptopt)
+            ddk_inp = inp.new_with_vars(optdriver=8, kptopt=kptopt)
             ddk_inp.set_spell_check(False)
             ddk_inp.set_vars(wfk_task="wfk_ddk")
             ddk_task = new.register_nscf_task(ddk_inp, deps={wfk_task: "WFK", den_file: "DEN"}, manager=tm)
@@ -1874,6 +2011,7 @@ class GKKPWork(Work):
                 wfq_path = os.path.join(os.path.dirname(wfk_path), infile)
                 if not os.path.isfile(wfq_path): os.symlink(wfk_path, wfq_path)
                 deps[FileNode(wfq_path)] = 'WFQ'
+
             new.register_eph_task(eph_input, deps=deps)
 
         return new
@@ -1905,6 +2043,8 @@ class GKKPWork(Work):
         return super().on_ok(sender)
 
 
+# TODO: We may deprecate it and use PhononWork that is more general!
+
 class BecWork(Work, MergeDdb):
     """
     Work for the computation of the Born effective charges.
@@ -1940,7 +2080,7 @@ class BecWork(Work, MergeDdb):
         the final DDB file in the outdir of the |Work|.
         """
         # Merge DDB files.
-        out_ddb = self.merge_ddb_files()
+        out_ddb = self.merge_ddb_files(only_dfpt_tasks=True)
         return self.Results(node=self, returncode=0, message="DDB merge done")
 
 
@@ -1955,7 +2095,7 @@ class DteWork(Work, MergeDdb):
     .. inheritance-diagram:: DteWork
     """
     @classmethod
-    def from_scf_task(cls, scf_task, ddk_tolerance=None, manager=None):
+    def from_scf_task(cls, scf_task, ddk_tolerance=None, manager=None) -> DteWork:
         """
         Build a DteWork from a ground-state task.
 
@@ -2010,7 +2150,7 @@ class DteWork(Work, MergeDdb):
         the final DDB file in the outdir of the `Work`.
         """
         # Merge DDB files.
-        out_ddb = self.merge_ddb_files()
+        out_ddb = self.merge_ddb_files(only_dfpt_tasks=True)
         return self.Results(node=self, returncode=0, message="DDB merge done")
 
 
@@ -2035,14 +2175,14 @@ class ConducWork(Work):
 
     @classmethod
     def from_phwork(cls, phwork, multi, nbr_proc=None, flow=None, with_kerange=False,
-                    omp_nbr_thread=1, manager=None):
+                    omp_nbr_thread=1, manager=None) -> ConducWork:
         """
         Construct a ConducWork from a |PhononWork| and |MultiDataset|.
 
         Args:
-            phwork: a |PhononWork| object calculating the DDB and DVDB files.
-            multi: a |MultiDataset| object containing a list of 3 datasets or 5 with Kerange.
-                       See abipy/abio/factories.py -> conduc_from_scf_nscf_inputs for details about multi.
+            phwork: |PhononWork| object calculating the DDB and DVDB files.
+            multi: |MultiDataset| object containing a list of 3 datasets or 5 with Kerange.
+                See abipy/abio/factories.py -> conduc_from_scf_nscf_inputs for details about multi.
             nbr_proc: Required if with_kerange since autoparal doesn't work with optdriver=8.
             flow: The flow calling the work. Used for  with_fixed_mpi_omp.
             with_kerange: True if using Kerange.
@@ -2073,7 +2213,7 @@ class ConducWork(Work):
         new.register_task(multi[1], deps={new[0]: "DEN"})
         taskNumber = 2 # To keep track of the task in new and multi
 
-        if(with_kerange): # Using Kerange
+        if with_kerange:  # Using Kerange
             new.register_task(multi[2], deps={new[1]: "WFK"})
             new.register_task(multi[3], deps={new[0]: "DEN", new[1]: "WFK", new[2]: "KERANGE.nc"})
             taskNumber = 4 # We have 2 more dataset
@@ -2109,7 +2249,7 @@ class ConducWork(Work):
             manager: |TaskManager| of the task. If None, the manager is initialized from the config file.
         """
         # Verify Multi
-        if (not with_kerange) and (multi.ndtset != 3): #Without kerange, multi should contain 4 datasets
+        if (not with_kerange) and (multi.ndtset != 3): # Without kerange, multi should contain 4 datasets
             raise ValueError("""The |MultiDataset| object does not contain the expected number of dataset.
                                 It should have 4 datasets and it had `%s`. You should generate
                                 multi with the factory function conduc_from_scf_nscf_inputs""" % multi.ndtset)
