@@ -39,5 +39,4 @@ class PhononBandsPlotterPanel(AbipyParameterized):
 
         if as_dict: return d
 
-        tabs = pn.Tabs(*d.items())
-        return self.get_template_from_tabs(tabs, template=kwargs.get("template", None))
+        return self.get_template_from_tabs(d, template=kwargs.get("template", None))
