@@ -377,8 +377,10 @@ def abicomp_phbands(options):
 
         # Select the plot method to call.
         if options.plot_mode == "panel":
-            #template=options.panel_template)
-            plotter.get_panel().show(debug=options.verbose > 0)
+            pn = abilab.abipanel()
+            serve_kwargs = serve_kwargs_from_options(options)
+            app = plotter.get_panel(template=options.panel_template)
+            return pn.serve(app, **serve_kwargs)
 
         elif options.plot_mode != "None":
             plotfunc = getattr(plotter, options.plot_mode, None)
@@ -640,16 +642,16 @@ def _invoke_robot(options):
                                      no_browser=options.no_browser)
 
     elif options.panel:
-        abilab.abipanel()
-
-        if hasattr(robot, "get_panel"):
-            app = robot.get_panel(template=options.panel_template)
-            app.show(debug=options.verbose > 0)
-            return 0
-        else:
+        if not hasattr(robot, "get_panel"):
             cprint(f"`{type(robot)} does not provide get_panel method", color="red")
             return 1
 
+        pn = abilab.abipanel()
+
+        serve_kwargs = serve_kwargs_from_options(options)
+        app = robot.get_panel(template=options.panel_template)
+        return pn.serve(app, **serve_kwargs)
+
     elif options.print or options.expose:
         robot.trim_paths()
         # Print dataframe if robot provides get_dataframe method.
@@ -1092,6 +1094,7 @@ the full set of atoms. Note that a value larger than 0.01 is considered to be un
                                    "Possible values are: FastList, FastGrid, Golden, Bootstrap, Material, React, Vanilla." +
                                    "Default: FastList"
                               )
+    robot_parser.add_argument("--port", default=0, type=int, help="Allows specifying a specific port when serving panel app.")
 
     robot_parents = [copts_parser, robot_ipy_parser, robot_parser, expose_parser, pandas_parser]
     p_gsr = subparsers.add_parser('gsr', parents=robot_parents, help=abicomp_gsr.__doc__)
@@ -1133,6 +1136,31 @@ the full set of atoms. Note that a value larger than 0.01 is considered to be un
     return parser
 
 
+def serve_kwargs_from_options(options):
+
+    #address = "localhost"
+    if options.no_browser:
+        print("""
+Use:
+
+    ssh -N -f -L localhost:{port}:localhost:{port} username@your_remote_cluster
+
+for port forwarding.
+""")
+
+    import abipy.panels as mod
+    assets_path = os.path.join(os.path.dirname(mod.__file__), "assets")
+
+    return dict(
+        debug=options.verbose > 0,
+        show=not options.no_browser,
+        port=options.port,
+        static_dirs={"/assets": assets_path},
+        #address=address,
+        #websocket_origin="{address}:{port}",
+    )
+
+
 @prof_main
 def main():
 
