@@ -7,6 +7,7 @@ import sys
 import os
 import argparse
 
+from pprint import pformat
 from monty.functools import prof_main
 from monty.termcolor import cprint
 from abipy import abilab
@@ -32,6 +33,11 @@ def df_to_clipboard(options, df):
         df.to_clipboard()
 
 
+class NegateAction(argparse.Action):
+    def __call__(self, parser, ns, values, option):
+        setattr(ns, self.dest, option[2:4] != 'no')
+
+
 #def abiview_fields(options):
 #    """Animate fields with Mayavi. Accept any file with density or potential ..."""
 #    from abipy.display.mvtk import MayaviFieldAnimator
@@ -151,7 +157,8 @@ def abiview_skw(options):
 
 def abiview_fs(options):
     """
-    Extract eigenvalues in the IBZ from file and visualize Fermi surface with --appname
+    Extract eigenvalues in the IBZ from file and visualize Fermi surface with
+    the external application specified via --appname
     """
     with abilab.abiopen(options.filepath) as abifile:
         eb3d = abifile.ebands.get_ebands3d()
@@ -168,6 +175,31 @@ def abiview_fs(options):
         return 0
 
 
+def abiview_ifermi_fs(options):
+    """
+    Use ifermi package to visualize the Fermi surface. Requires netcdf file with energies in the IBZ.
+    See <https://fermisurfaces.github.io/IFermi/>
+    """
+    with abilab.abiopen(options.filepath) as abifile:
+
+        kwargs = dict(
+            interpolation_factor=options.interpolation_factor,
+            mu=options.mu,
+            eref=options.eref,
+            wigner_seitz=options.wigner,
+            with_velocities=options.with_velocities,
+        )
+        print(abifile.ebands, "\n")
+        print("Building Fermi surface with options:\n\n", pformat(kwargs, indent=4), end=2*"\n")
+
+        r = abifile.ebands.get_ifermi_fs(**kwargs)
+        r.fs_plotter.get_plot(plot_type=options.plot_type).show()
+
+        #abifile.ebands.get_ifermi_slices(**kwargs)
+
+        return 0
+
+
 def abiview_ddb(options):
     """
     Invoke Anaddb to compute phonon bands and DOS from the DDB, plot the results.
@@ -372,6 +404,29 @@ qpoint = {qpoint}
     return 0
 
 
+def abiview_ddb_qpt(options):
+    """
+    Compute ph-frequencies at the selected q-point without passing through the
+    Fourier interpolation of the interatomic force constants.
+    """
+    #    asr = 2; chneut = 1; dipdip = 1
+    #    print("""
+    #Computing interatomic force constants with
+    #asr: {asr}, chneut: {chneut}, dipdip: {dipdip}
+    #""".format(**locals()))
+
+    with abilab.abiopen(options.filepath) as ddb:
+        # Execute anaddb to compute phbands without Fourier interpolation.
+        phbands, inp = ddb.anaget_phmodes_at_qpoint(qpoint=options.qpoint,
+                            asr=2, chneut=1, dipdip=1, ifcflag=0, return_input=True)
+        print(inp)
+        df = phbands.get_dataframe()
+        abilab.print_dataframe(df, title="Phonon frequencies:")
+        df_to_clipboard(options, df)
+
+    return 0
+
+
 def abiview_ddb_ifc(options):
     """
     Visualize interatomic force constants in real space.
@@ -507,12 +562,14 @@ Usage example:
     abiview.py ddb_asr                    ==>  Compute ph-bands from DDB with/wo acoustic rule.
                                                Plot results with matplotlib (default) or plotly (--plotly)
     abiview.py ddb_asr --plotly -cs       ==>  Compute ph-bands from DDB with/wo acoustic rule.
-                                               Plot results with plotly and push figure to plotly chart studio cloud.
-                                               See: https://chart-studio.plotly.com/
+                                               Plot results with plotly and push figure
+                                               to plotly chart studio cloud. See: https://chart-studio.plotly.com
     abiview.py ddb_dipdip                 ==>  Compute ph-bands from DDB with/wo dipole-dipole treatment.
                                                Plot results with matplotlib (default) or plotly (--plotly)
     abiview.py ddb_quad                   ==>  Compute ph-bands from DDB with/wo dipole-quadrupole terms.
                                                Plot results with matplotlib (default) or plotly (--plotly)
+    abiview.py ddb_qpt -q 0 0.5 0         ==>  Compute ph-frequencies at the selected q-point without passing
+                                               through the Fourier interpolation of the interatomic force constants.
     abiview.py ddb_ifc                    ==>  Visualize interatomic force constants in real space.
     abiview.py phbands out_PHBST.nc -web  ==>  Visualize ph-bands and displacements with phononwebsite.
 
@@ -661,6 +718,26 @@ def get_parser(with_epilog=False):
     p_fs.add_argument("-a", "--appname", type=str, default="mpl",
         help="Application name. Possible options: mpl (matplotlib, default), xsf (xcrysden), mayavi.")
 
+    # Parent parser for ifermi commands
+    ifermi_parser = argparse.ArgumentParser(add_help=False)
+    ifermi_parser.add_argument("-i", '--interpolation-factor', default=8, type=float,
+        help="interpolation factor for band structure  [default: 8.0]")
+    ifermi_parser.add_argument("--eref", default="fermie", type=str, choices=['fermie', 'cbm', 'vbm'],
+        help="Energy reference for isosurface. `eref` and `mu` define the energy level: isoe = eref + mu"  +
+              "Use `fermie` for metals, `cbm` for the conduction band minimum, and `vbm` for the valence band maximum, " +
+             "[default: `fermie`]")
+    ifermi_parser.add_argument("-t", '--plot-type', default="plotly", choices=["plotly", "matplotlib", "mayavi"],
+        help="Plot type. Possible options: plotly (default), matplotlib, mayavi.")
+    ifermi_parser.add_argument('--wigner', '--no-wigner', dest='wigner', default=True, action=NegateAction, nargs=0,
+            help="Use the Wigner-Seitz cell or the reciprocal lattice parallelepiped. Default is: wigner")
+    ifermi_parser.add_argument("-vel", '--with-velocities', default=False, action="store_true",
+            help="Show color map with interpolated group velocities. Default is False.")
+
+    # Subparser for ifermi_fs command.
+    p_ifermi_fs = subparsers.add_parser('ifermi_fs', parents=[ifermi_parser, copts_parser], help=abiview_ifermi_fs.__doc__)
+    p_ifermi_fs.add_argument("-m", '--mu', default=0.0, type=float,
+        help="Offset in eV from the energy reference, eref, at which to calculate the isosurface. Default 0 i.e. use `eref`")
+
     # Subparser for ddb command.
     p_ddb = subparsers.add_parser('ddb', parents=[copts_parser, slide_parser, plotly_parser], help=abiview_ddb.__doc__)
     add_args(p_ddb, "xmgrace", "phononweb", "browser", "force")
@@ -693,7 +770,14 @@ def get_parser(with_epilog=False):
 
     # Subparser for ddb_ifc command.
     p_ddb_ifc = subparsers.add_parser('ddb_ifc', parents=[copts_parser, pandas_parser, slide_parser],
-                                     help=abiview_ddb_ifc.__doc__)
+                                      help=abiview_ddb_ifc.__doc__)
+
+    # Subparser for ddb_qpt command.
+    p_ddb_qpt = subparsers.add_parser('ddb_qpt', parents=[copts_parser, pandas_parser, slide_parser],
+                                      help=abiview_ddb_qpt.__doc__)
+
+    p_ddb_qpt.add_argument("-q", "--qpoint", nargs=3, type=float,
+        help="q-point in reduced coordinates e.g. `-q 0.25 0 0`. Default: 0, 0, 0", default=[0, 0, 0])
 
     # Subparser for phbands command.
     p_phbands = subparsers.add_parser('phbands', parents=[copts_parser, slide_parser], help=abiview_phbands.__doc__)
