@@ -1,11 +1,15 @@
 # coding: utf-8
 """Duck-typing tests"""
+from __future__ import annotations
+
 import collections
 import warnings
 import numpy as np
 
+from typing import Any
 
-def is_string(s):
+
+def is_string(s: Any) -> bool:
     """True if s behaves like a string (duck typing test)."""
     try:
         s + " "
@@ -14,7 +18,7 @@ def is_string(s):
         return False
 
 
-def is_intlike(obj):
+def is_intlike(obj: Any) -> bool:
     """
     True if obj represents an integer (float such as 1.0 are included as well).
     """
@@ -34,7 +38,7 @@ def is_intlike(obj):
     return False
 
 
-def is_number_like(obj):
+def is_number_like(obj: Any) -> bool:
     """True if obj represents a number."""
     try:
         obj - 1
@@ -43,7 +47,7 @@ def is_number_like(obj):
         return False
 
 
-def is_listlike(obj):
+def is_listlike(obj: Any) -> bool:
     #if isinstance(branch, (list, tuple, np.ndarray)):
     if isinstance(obj, np.ndarray): return True
     if not isinstance(obj, collections.abc.Sequence): return False
@@ -56,7 +60,7 @@ def is_listlike(obj):
         return False
 
 
-def list_ints(arg):
+def list_ints(arg: Any) -> list:
     """
     Always return a list of int, given a int or list of integers as input.
 
@@ -69,7 +73,7 @@ def list_ints(arg):
     return [int(l)] if l.size == 1 else l.tolist()
 
 
-def torange(obj):
+def torange(obj: Any) -> range:
     """
     Convert obj into a range. Accepts integer, slice object  or any object
     with an __iter__ method. Note that an integer is converted into range(int, int+1)
@@ -96,7 +100,7 @@ def torange(obj):
             raise TypeError("Don't know how to convert %s into a range object" % str(obj))
 
 
-def as_slice(obj):
+def as_slice(obj: Any) -> slice:
     """
     Convert an integer, a string or a slice object into slice.
 
@@ -134,7 +138,7 @@ class NoDefaultProvided(object):
     pass
 
 
-def hasattrd(obj, name):
+def hasattrd(obj: Any, name: str) -> bool:
     """
     The arguments are an object and a string.
     The result is True if the string is the name of one of the object’s attributes, False if not.
@@ -148,7 +152,7 @@ def hasattrd(obj, name):
         return False
 
 
-def getattrd(obj, name, default=NoDefaultProvided):
+def getattrd(obj: Any, name: str, default=NoDefaultProvided) -> Any:
     """
     Same as getattr(), but allows dot notation lookup e.g. getattrd(obj, "a.b")
 
