@@ -6,18 +6,22 @@ Utilities for generating matplotlib plots.
 
     Avoid importing matplotlib or plotly in the module namespace otherwise startup is very slow.
 """
+from __future__ import annotations
+
 import os
 import time
 import itertools
 import numpy as np
 
 from collections import OrderedDict, namedtuple
+from typing import Any, List
 from pymatgen.util.plotting import add_fig_kwargs, get_ax_fig_plt, get_ax3d_fig_plt, get_axarray_fig_plt
 from .numtools import data_from_cplx_mode
 
 
 __all__ = [
     "set_axlims",
+    "add_fig_kwargs",
     "get_ax_fig_plt",
     "get_ax3d_fig_plt",
     "plot_array",
@@ -27,6 +31,9 @@ __all__ = [
     "plot_unit_cell",
     "GenericDataFilePlotter",
     "GenericDataFilesPlotter",
+    "add_plotly_fig_kwargs",
+    "get_fig_plotly",
+    "get_figs_plotly",
 ]
 
 
@@ -55,13 +62,13 @@ linestyles = OrderedDict(
 # Matplotlib tools
 ###################
 
-def is_mpl_figure(obj):
+def is_mpl_figure(obj: Any) -> bool:
     """Return True if obj is a matplotlib Figure."""
     from matplotlib import pyplot as plt
     return isinstance(obj, plt.Figure)
 
 
-def ax_append_title(ax, title, loc="center", fontsize=None):
+def ax_append_title(ax, title, loc="center", fontsize=None) -> str:
     """Add title to previous ax.title. Return new title."""
     prev_title = ax.get_title(loc=loc)
     new_title = prev_title + title
@@ -69,7 +76,7 @@ def ax_append_title(ax, title, loc="center", fontsize=None):
     return new_title
 
 
-def ax_share(xy_string, *ax_list):
+def ax_share(xy_string: str, *ax_list) -> None:
     """
     Share x- or y-axis of two or more subplots after they are created
 
@@ -93,15 +100,7 @@ def ax_share(xy_string, *ax_list):
             ax.get_shared_y_axes().join(*others)
 
 
-#def set_grid(fig, boolean):
-#    if hasattr(fig, "axes"):
-#        for ax in fig.axes:
-#            if ax.grid: ax.grid.set_visible(boolean)
-#    else:
-#            if ax.grid: ax.grid.set_visible(boolean)
-
-
-def set_axlims(ax, lims, axname):
+def set_axlims(ax, lims, axname: str) -> tuple:
     """
     Set the data limits for the axis ax.
 
@@ -112,7 +111,7 @@ def set_axlims(ax, lims, axname):
     Return: (left, right)
     """
     left, right = None, None
-    if lims is None: return (left, right)
+    if lims is None: return left, right
 
     len_lims = None
     try:
@@ -134,7 +133,7 @@ def set_axlims(ax, lims, axname):
     return left, right
 
 
-def set_ax_xylabels(ax, xlabel, ylabel, exchange_xy):
+def set_ax_xylabels(ax, xlabel: str, ylabel: str, exchange_xy: bool) -> None:
     """
     Set the x- and the y-label of axis ax, exchanging x and y if exchange_xy
     """
@@ -143,7 +142,7 @@ def set_ax_xylabels(ax, xlabel, ylabel, exchange_xy):
     ax.set_ylabel(ylabel)
 
 
-def set_visible(ax, boolean, *args):
+def set_visible(ax, boolean: bool, *args) -> None:
     """
     Hide/Show the artists of axis ax listed in args.
     """
@@ -163,7 +162,7 @@ def set_visible(ax, boolean, *args):
             label.set_visible(boolean)
 
 
-def rotate_ticklabels(ax, rotation, axname="x"):
+def rotate_ticklabels(ax, rotation: float, axname: str ="x") -> None:
     """Rotate the ticklables of axis ``ax``"""
     if "x" in axname:
         for tick in ax.get_xticklabels():
@@ -187,7 +186,7 @@ def plot_xy_with_hue(data, x, y, hue, decimals=None, ax=None,
         hue: Variable that define subsets of the data, which will be drawn on separate lines
         decimals: Number of decimal places to round `hue` columns. Ignore if None
         ax: |matplotlib-Axes| or None if a new figure should be created.
-        xlims ylims: Set the data limits for the x(y)-axis. Accept tuple e.g. `(left, right)`
+        xlims, ylims: Set the data limits for the x(y)-axis. Accept tuple e.g. `(left, right)`
             or scalar e.g. `left`. If left (right) is None, default values are used
         fontsize: Legend fontsize.
         kwargs: Keywork arguments are passed to ax.plot method.
@@ -289,7 +288,7 @@ def plot_array(array, color_map=None, cplx_mode="abs", **kwargs):
     return fig
 
 
-class ArrayPlotter(object):
+class ArrayPlotter:
 
     def __init__(self, *labels_and_arrays):
         """
@@ -460,7 +459,7 @@ class Marker(namedtuple("Marker", "x y s")):
         return self.__class__(pos_x, pos_y, pos_s), Marker(neg_x, neg_y, neg_s)
 
 
-class MplExpose(object): # pragma: no cover
+class MplExpose: # pragma: no cover
     """
     Context manager used to produce several matplotlib figures and then show
     all them at the end so that the user does not need to close the window to
@@ -544,7 +543,7 @@ class MplExpose(object): # pragma: no cover
                 fig.clear()
 
 
-class PanelExpose(object): # pragma: no cover
+class PanelExpose:  # pragma: no cover
     """
     Context manager used to produce several matplotlib/plotly figures and then show
     all them inside the Browser using a panel template.
@@ -809,7 +808,7 @@ class GenericDataFilePlotter(object):
     def __str__(self):
         return self.to_string()
 
-    def to_string(self, verbose=0):
+    def to_string(self, verbose: int = 0) -> str:
         """String representation with verbosity level `verbose`."""
         lines = []
         for key, arr in self.od.items():
@@ -852,10 +851,10 @@ class GenericDataFilePlotter(object):
         return fig
 
 
-class GenericDataFilesPlotter(object):
+class GenericDataFilesPlotter:
 
     @classmethod
-    def from_files(cls, filepaths):
+    def from_files(cls, filepaths: List[str]) -> GenericDataFilesPlotter:
         """
         Build object from a list of `filenames`.
         """
@@ -871,7 +870,7 @@ class GenericDataFilesPlotter(object):
     def __str__(self):
         return self.to_string()
 
-    def to_string(self, verbose=0):
+    def to_string(self, verbose: int = 0) -> str:
         lines = []
         app = lines.append
         for od, filepath in zip(self.odlist, self.filepaths):
@@ -881,7 +880,7 @@ class GenericDataFilesPlotter(object):
 
         return "\n".join(lines)
 
-    def add_file(self, filepath):
+    def add_file(self, filepath: str) -> None:
         """Add data from `filepath`"""
         with open(filepath, "rt") as fh:
             self.odlist.append(_generic_parser_fh(fh))
@@ -953,14 +952,77 @@ class GenericDataFilesPlotter(object):
 # Plotly helper functions
 ##########################
 
-def is_plotly_figure(obj):
+_LATEX_GREEK_TO_UNICODE = dict(
+    alpha="α",
+    beta="β",
+    gamma="ɣ",
+    delta="δ",
+    epsilon="ε",
+    zeta="ζ",
+    eta="η",
+    theta="θ",
+    iota="ι",
+    kappa="κ",
+    #lambda="λ",
+    mu="μ",
+    nu="ν",
+    xi="ξ",
+    omicron="ο",
+    pi="π",
+    rho="ρ",
+    sigma="σ",
+    tau="τ",
+    upsilon="υ",
+    phi="φ",
+    chi="χ",
+    psi="ψ",
+    omega="ω",
+    # Capital case:
+    Alpha="Α",
+    Beta="Β",
+    Gamma="Γ",
+    Delta="Δ",
+    Epsilon="Ε",
+    Zeta="Ζ",
+    Eta="Η",
+    Theta="Θ",
+    Iota="Ι",
+    Kappa="Κ",
+    Lambda="Λ",
+    Mu="Μ",
+    Nu="Ν",
+    Xi="Ξ",
+    Omicron="Ο",
+    Po="Π",
+    Rho="Ρ",
+    Sigma="Σ",
+    Tau="Τ",
+    Upsilon="Υ",
+    Phi="Φ",
+    Chi="Χ",
+    Psi="Ψ",
+    Omega="Ω",
+)
+
+_LATEX_GREEK_TO_UNICODE["lambda"] = "λ"
+
+
+def latex_greek_2unicode(latex: str) -> str:
+    """
+    Convert a single greek letter in latex notation into unicode
+    """
+    s = latex.replace("$", "").replace("\\", "").strip()
+    return _LATEX_GREEK_TO_UNICODE[s]
+
+
+def is_plotly_figure(obj: Any) -> bool:
     """Return True if obj is a plotly Figure."""
     import plotly.graph_objs as go
     return isinstance(obj, go.Figure)
     #return isinstance(obj, (go.Figure, go.FigureWidget))
 
 
-class PlotlyRowColDesc(object):
+class PlotlyRowColDesc:
     """
     This object specifies the position of a plotly subplot inside a grid.
 
@@ -968,7 +1030,7 @@ class PlotlyRowColDesc(object):
     """
 
     @classmethod
-    def from_object(cls, obj):
+    def from_object(cls, obj: Any) -> PlotlyRowColDesc:
         """
         Build an instance for a generic object.
         If oject is None, a simple descriptor corresponding to a (1,1) grid is returned.
@@ -982,7 +1044,7 @@ class PlotlyRowColDesc(object):
         except Exception as exc:
             raise TypeError(f"Dont know how to convert `{type(obj)}` into `{cls}`")
 
-    def __init__(self, py_row, py_col, nrows, ncols):
+    def __init__(self, py_row: int, py_col: int, nrows: int, ncols: int):
         """
         Args:
             py_row, py_col: python index of the subplot in the grid (starts from 0)
@@ -997,7 +1059,7 @@ class PlotlyRowColDesc(object):
         else:
             self.ply_row, self.ply_col = (self.py_row + 1, self.py_col + 1)
 
-    def __str__(self):
+    def __str__(self) -> str:
         lines = []
         app = lines.append
         app("py_rowcol: (%d, %d) in grid: (%d, %d)" % (self.py_row, self.py_col, self.nrows, self.ncols))
@@ -1022,7 +1084,7 @@ def get_figs_plotly(nrows=1, ncols=1, subplot_titles=(), sharex=False, sharey=Fa
     from plotly.subplots import make_subplots
     import plotly.graph_objects as go
 
-    fig = make_subplots(rows=nrows, cols=ncols, subplot_titles=subplot_titles, shared_xaxes=sharex,
+    fig = make_subplots(rows=int(nrows), cols=int(ncols), subplot_titles=subplot_titles, shared_xaxes=sharex,
                         shared_yaxes=sharey, **fig_kw)
 
     return fig, go
@@ -1051,6 +1113,7 @@ def plotly_set_lims(fig, lims, axname, iax=None):
     Set the data limits for the axis ax.
 
     Args:
+        fig: Plotly Figure.
         lims: tuple(2) for (left, right), if tuple(1) or scalar for left only, none is set.
         axname: "x" for x-axis, "y" for y-axis.
         iax: An int, use iax=n to decorate the nth axis when the fig has subplots.
@@ -1098,7 +1161,7 @@ def plotly_set_lims(fig, lims, axname, iax=None):
 _PLOTLY_DEFAULT_SHOW = [True]
 
 
-def set_plotly_default_show(true_or_false):
+def set_plotly_default_show(true_or_false: bool) -> None:
     """
     Set the default value of show in the add_plotly_fig_kwargs decorator.
     Usefule for instance when generating the sphinx gallery of plotly plots.
@@ -1126,6 +1189,7 @@ def add_plotly_fig_kwargs(func):
         config = kwargs.pop("config", None)
         renderer = kwargs.pop("renderer", None)
         chart_studio = kwargs.pop("chart_studio", False)
+        template = kwargs.pop("template", None)
 
         # Allow users to specify the renderer via shell env.
         if renderer is not None and os.getenv("PLOTLY_RENDERER", default=None) is not None:
@@ -1140,8 +1204,29 @@ def add_plotly_fig_kwargs(func):
         if title is not None:
             fig.update_layout(title_text=title, title_x=0.5)
 
+        if template is not None:
+            fig.update_layout(template=template)
+
         if savefig:
-            fig.write_image(savefig)
+            # https://plotly.github.io/plotly.py-docs/generated/plotly.io.write_image.html
+            if savefig.endswith("html"):
+                from plotly.offline import plot as show_plotly
+                show_plotly(fig, include_mathjax="cdn", filename=savefig, auto_open=False)
+
+            else:
+                try:
+                    import kaleido
+                except ImportError:
+                    kaleido = False
+
+                if kaleido is None:
+                    raise ValueError(
+                        "kaleido package required to save static ploty images\n"
+                        "please install it using:\npip install kaleido"
+                    )
+
+                fig.write_image(savefig, engine="kaleido", scale=5, width=750, height=750)
+                #fig.write_image(savefig)
 
         if write_json:
             import plotly.io as pio
@@ -1150,7 +1235,19 @@ def add_plotly_fig_kwargs(func):
         fig.layout.hovermode = hovermode
 
         if show: # and _PLOTLY_DEFAULT_SHOW:
-            fig.show(renderer=renderer, config=config)
+            my_config = dict(
+                responsive=True,
+                #showEditInChartStudio=True,
+                showLink=True,
+                plotlyServerURL="https://chart-studio.plotly.com",
+            )
+
+            if config is not None:
+                my_config.update(config)
+
+            #add_template_buttons(fig)
+
+            fig.show(renderer=renderer, config=my_config)
 
         if chart_studio:
             push_to_chart_studio(fig)
@@ -1183,6 +1280,9 @@ def add_plotly_fig_kwargs(func):
                 config (dict)     A dict of parameters to configure the figure. The defaults are set in plotly.js.
                 chart_studio      True to push figure to chart_studio server. Requires authenticatios.
                                   Default: False.
+                template          Plotly template. See https://plotly.com/python/templates/
+                                  ["plotly", "plotly_white", "plotly_dark", "ggplot2", "seaborn", "simple_white", "none"]
+                                  Default is None that is the default template is used.
                 ================  ====================================================================
         """
     )
@@ -1329,7 +1429,7 @@ def push_to_chart_studio(figs):
 ####################################################
 # This code is shamelessy taken from Adam's package
 ####################################################
-import plotly.graph_objects as go
+#import plotly.graph_objects as go
 
 
 def go_points(points, size=4, color="black", labels=None, **kwargs):
@@ -1342,6 +1442,7 @@ def go_points(points, size=4, color="black", labels=None, **kwargs):
     if labels is not None:
         labels = plotly_klabels(labels, allow_dupes=True)
 
+    import plotly.graph_objects as go
     return go.Scatter3d(
         x=[v[0] for v in points],
         y=[v[1] for v in points],
@@ -1363,6 +1464,7 @@ def go_line(v1, v2, color="black", width=2, mode="lines", **kwargs):
     _add_if_not_in(kwargs, "line_color", "black")
     _add_if_not_in(kwargs, "line_width", 2)
 
+    import plotly.graph_objects as go
     return go.Scatter3d(
         mode=mode,
         x=[v1[0], v2[0]],
@@ -1374,6 +1476,7 @@ def go_line(v1, v2, color="black", width=2, mode="lines", **kwargs):
 
 
 def go_lines(V, name=None, color="black", width=2, **kwargs):
+    import plotly.graph_objects as go
     gen = ((v1, v2) for (v1, v2) in V)
     v1, v2 = next(gen)
     out = [
@@ -1467,6 +1570,7 @@ def plot_fcc_conv():
         legendgroup="atoms",
     )
 
+    import plotly.graph_objects as go
     fig = go.Figure(data=[*fcc_box, *fcc_vectors, atoms])
     return fig
 
@@ -1490,6 +1594,7 @@ def plot_fcc_prim():
     fcc_conv = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
     fcc_conv_box = get_box(fcc_conv, name="conv lattice")
 
+    import plotly.graph_objects as go
     fig = go.Figure(data=[*fcc_prim_box, *fcc_prim_vectors, *fcc_conv_box, atoms])
 
     return fig
@@ -1609,7 +1714,7 @@ def plotly_structure(structure, ax=None, to_unit_cell=False, alpha=0.7,
     #)
 
     # The definition of sizes is not optimal because matplotlib uses points
-    # wherease we would like something that depends on the radius (5000 seems to give reasonable plots)
+    # whereas we would like something that depends on the radius (5000 seems to give reasonable plots)
     # For possibile approaches, see
     # https://stackoverflow.com/questions/9081553/python-scatter-plot-size-and-style-of-the-marker/24567352#24567352
     # https://gist.github.com/syrte/592a062c562cd2a98a83
@@ -1622,6 +1727,7 @@ def plotly_structure(structure, ax=None, to_unit_cell=False, alpha=0.7,
     #ax.set_axis_off()
 
     #fig = go.Figure(data=[*box, *vectors, atoms])
+    import plotly.graph_objects as go
     fig = go.Figure(data=[*box, atoms])
     return fig
 
@@ -1667,7 +1773,7 @@ def plotly_wigner_seitz(lattice, fig=None, **kwargs):
 
 def plotly_lattice_vectors(lattice, fig=None, **kwargs):
     """
-    Adds the basis vectors of the lattice provided to a matplotlib Axes
+    Adds the basis vectors of the lattice provided to a plotly figure.
 
     Args:
         lattice: Lattice object
@@ -1675,11 +1781,9 @@ def plotly_lattice_vectors(lattice, fig=None, **kwargs):
         kwargs: kwargs passed to the matplotlib function 'plot'. Color defaults to green
             and linewidth to 3.
 
-    Returns:
-        matplotlib figure and matplotlib ax
+    Returns: plotly figure
     """
-    #ax, fig, plt = get_ax3d_fig_plt(ax)
-    fig, go = get_fig_plotly(fig=fig) #, **fig_kw)
+    fig, go = get_fig_plotly(fig=fig)
 
     if "line_color" not in kwargs:
         kwargs["line_color"] = "green"
@@ -1690,13 +1794,10 @@ def plotly_lattice_vectors(lattice, fig=None, **kwargs):
 
     vertex1 = lattice.get_cartesian_coords([0.0, 0.0, 0.0])
     vertex2 = lattice.get_cartesian_coords([1.0, 0.0, 0.0])
-    #ax.plot(*zip(vertex1, vertex2), **kwargs)
     fig.add_trace(go_line(vertex1, vertex2, name="a", **kwargs))
     vertex2 = lattice.get_cartesian_coords([0.0, 1.0, 0.0])
-    #ax.plot(*zip(vertex1, vertex2), **kwargs)
     fig.add_trace(go_line(vertex1, vertex2, name="b", **kwargs))
     vertex2 = lattice.get_cartesian_coords([0.0, 0.0, 1.0])
-    #ax.plot(*zip(vertex1, vertex2), **kwargs)
     fig.add_trace(go_line(vertex1, vertex2, name="c", **kwargs))
 
     return fig
@@ -1704,7 +1805,7 @@ def plotly_lattice_vectors(lattice, fig=None, **kwargs):
 
 def plotly_path(line, lattice=None, coords_are_cartesian=False, fig=None, **kwargs):
     """
-    Adds a line passing through the coordinates listed in 'line' to a matplotlib Axes
+    Adds a line passing through the coordinates listed in 'line' to a plotly figure.
 
     Args:
         line: list of coordinates.
@@ -1716,12 +1817,9 @@ def plotly_path(line, lattice=None, coords_are_cartesian=False, fig=None, **kwar
         kwargs: kwargs passed to the matplotlib function 'plot'. Color defaults to red
             and linewidth to 3.
 
-    Returns:
-        matplotlib figure and matplotlib ax
+    Returns: plotly figure
     """
-
-    #ax, fig, plt = get_ax3d_fig_plt(ax)
-    fig, go = get_fig_plotly(fig=fig) #, **fig_kw)
+    fig, go = get_fig_plotly(fig=fig)
 
     if "line_color" not in kwargs:
         kwargs["line_color"] = "red"
@@ -1736,55 +1834,55 @@ def plotly_path(line, lattice=None, coords_are_cartesian=False, fig=None, **kwar
                 raise ValueError("coords_are_cartesian False requires the lattice")
             vertex1 = lattice.get_cartesian_coords(vertex1)
             vertex2 = lattice.get_cartesian_coords(vertex2)
-        #ax.plot(*zip(vertex1, vertex2), **kwargs)
+
         fig.add_trace(go_line(vertex1, vertex2, showlegend=False, **kwargs))
 
     return fig
 
 
-def plotly_labels(labels, lattice=None, coords_are_cartesian=False, ax=None, **kwargs):
-    """
-    Adds labels to a matplotlib Axes
-
-    Args:
-        labels: dict containing the label as a key and the coordinates as value.
-        lattice: Lattice object used to convert from reciprocal to cartesian coordinates
-        coords_are_cartesian: Set to True if you are providing.
-            coordinates in cartesian coordinates. Defaults to False.
-            Requires lattice if False.
-        ax: matplotlib :class:`Axes` or None if a new figure should be created.
-        kwargs: kwargs passed to the matplotlib function 'text'. Color defaults to blue
-            and size to 25.
-
-    Returns:
-        matplotlib figure and matplotlib ax
-    """
-    ax, fig, plt = get_ax3d_fig_plt(ax)
-
-    if "color" not in kwargs:
-        kwargs["color"] = "b"
-    if "size" not in kwargs:
-        kwargs["size"] = 25
-
-    for k, coords in labels.items():
-        label = k
-        if k.startswith("\\") or k.find("_") != -1:
-            label = "$" + k + "$"
-        off = 0.01
-        if coords_are_cartesian:
-            coords = np.array(coords)
-        else:
-            if lattice is None:
-                raise ValueError("coords_are_cartesian False requires the lattice")
-            coords = lattice.get_cartesian_coords(coords)
-        ax.text(*(coords + off), s=label, **kwargs)
-
-    return fig, ax
+#def plotly_labels(labels, lattice=None, coords_are_cartesian=False, ax=None, **kwargs):
+#    """
+#    Adds labels to a matplotlib Axes
+#
+#    Args:
+#        labels: dict containing the label as a key and the coordinates as value.
+#        lattice: Lattice object used to convert from reciprocal to cartesian coordinates
+#        coords_are_cartesian: Set to True if you are providing.
+#            coordinates in cartesian coordinates. Defaults to False.
+#            Requires lattice if False.
+#        ax: matplotlib :class:`Axes` or None if a new figure should be created.
+#        kwargs: kwargs passed to the matplotlib function 'text'. Color defaults to blue
+#            and size to 25.
+#
+#    Returns:
+#        matplotlib figure and matplotlib ax
+#    """
+#    ax, fig, plt = get_ax3d_fig_plt(ax)
+#
+#    if "color" not in kwargs:
+#        kwargs["color"] = "b"
+#    if "size" not in kwargs:
+#        kwargs["size"] = 25
+#
+#    for k, coords in labels.items():
+#        label = k
+#        if k.startswith("\\") or k.find("_") != -1:
+#            label = "$" + k + "$"
+#        off = 0.01
+#        if coords_are_cartesian:
+#            coords = np.array(coords)
+#        else:
+#            if lattice is None:
+#                raise ValueError("coords_are_cartesian False requires the lattice")
+#            coords = lattice.get_cartesian_coords(coords)
+#        ax.text(*(coords + off), s=label, **kwargs)
+#
+#    return fig, ax
 
 
 def plotly_points(points, lattice=None, coords_are_cartesian=False, fold=False, labels=None, fig=None, **kwargs):
     """
-    Adds points to a matplotlib Axes
+    Adds points to a plotly figure.
 
     Args:
         points: list of coordinates
@@ -1797,10 +1895,8 @@ def plotly_points(points, lattice=None, coords_are_cartesian=False, fold=False,
         fig: plotly figure or None if a new figure should be created.
         kwargs: kwargs passed to the matplotlib function 'scatter'. Color defaults to blue
 
-    Returns:
-        matplotlib figure and matplotlib ax
+    Returns: plotly figure
     """
-    #ax, fig, plt = get_ax3d_fig_plt(ax)
     fig, go = get_fig_plotly(fig=fig) #, **fig_kw)
 
     if "marker_color" not in kwargs:
@@ -1820,7 +1916,6 @@ def plotly_points(points, lattice=None, coords_are_cartesian=False, fold=False,
             p = lattice.get_cartesian_coords(p)
 
         vecs.append(p)
-        #ax.scatter(*p, **kwargs)
 
     kws = dict(textposition="top right", showlegend=False) #, textfont=dict(color='#E58606'))
     kws.update(kwargs)
@@ -1848,6 +1943,7 @@ def plotly_brillouin_zone_from_kpath(kpath, fig=None, **kwargs):
         lines=lines,
         fig=fig,
         labels=kpath.kpath["kpoints"],
+        show=False,
         **kwargs,
     )
 
@@ -1916,3 +2012,78 @@ def plotly_brillouin_zone(
     #ax.axis("off")
 
     return fig
+
+
+def add_colorscale_dropwdowns(fig):
+    """
+    Add dropdown widgets to change/reverse the colorscale.
+    Based on: https://plotly.com/python/dropdowns/#update-several-data-attributes
+    """
+    button_layer_1_height = 1.30
+
+    # Create list of buttons
+    # A single button has the form:
+    #
+    #    dict(
+    #        args=["colorscale", "Viridis"],
+    #        label="Viridis",
+    #        method="restyle"
+    #    ),
+
+    colorscales = ["Viridis", "Cividis", "Blues", "Greens"]
+
+    colorscale_buttons = []
+    for cscale in colorscales:
+        colorscale_buttons.append(dict(
+                args=["colorscale", cscale],
+                label=cscale,
+                method="restyle",
+        ))
+
+    fig.update_layout(
+        updatemenus=[
+            dict(
+                buttons=colorscale_buttons,
+                direction="down",
+                pad={"r": 10, "t": 10},
+                showactive=True,
+                x=0.1,
+                xanchor="left",
+                y=button_layer_1_height,
+                yanchor="top"
+            ),
+            dict(
+                buttons=list([
+                    dict(
+                        args=["reversescale", False],
+                        label="False",
+                        method="restyle"
+                    ),
+                    dict(
+                        args=["reversescale", True],
+                        label="True",
+                        method="restyle"
+                    )
+                ]),
+                direction="down",
+                pad={"r": 10, "t": 10},
+                showactive=True,
+                x=0.37,
+                xanchor="left",
+                y=button_layer_1_height,
+                yanchor="top"
+            ),
+        ]
+    )
+
+    y = button_layer_1_height - 0.02
+
+    fig.update_layout(
+        annotations=[
+            dict(text="colorscale", x=0, xref="paper", y=y, yref="paper",
+                 align="left", showarrow=False),
+            dict(text="Reverse<br>Colorscale", x=0.25, xref="paper", y=y,
+                 yref="paper", showarrow=False),
+    ])
+
+    return fig
